/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "my_stdio.h"

#include "camera.h"
#include "fits_io.h"
#include "camera_tools.h"
#include "version.h"
#include "novas.h"
#include "ang_tools.h"

char *local_config_file;

site_info geo_loc;


int set_location(void)
{
	struct image_2d im;
	int ret;
	char cmd[_POSIX_PATH_MAX];
	char *editor;

	local_config_file = (char *) malloc(_POSIX_PATH_MAX * sizeof(char));
	sprintf(local_config_file,"%s/%s", getenv("HOME"), CONFIG_FILE);
	im.ft = (struct fit_tag *) malloc(sizeof(struct fit_tag));
	im.ft->next = NULL;
	ct_read_header_file(&im, GLOBAL_CONFIG_FILE);
	do {
		ret = ct_read_header_file(&im, local_config_file);
		if (ret == -1) {
			sprintf(cmd, "cp %s %s", USER_DEF_CONFIG_FILE,
				local_config_file);
			system(cmd);
			editor = getenv("EDITOR");
			if( editor == NULL) editor = DEF_EDITOR;
			sprintf(cmd,"%s  %s", editor, local_config_file);
			system(cmd);
		}
	} while(ret == -1);
	geo_loc.latitude = im.latitude;
	geo_loc.longitude = im.longitude;
	geo_loc.height = im.altitude;
	geo_loc.temperature = 20.0; /* not used */
	geo_loc.pressure = 1020.0; /* not used */
	fits_tag_free(im.ft);

	return 0;
}


int get_object_position(double *ra, double *dec)
{
	
	struct image_2d im;
	struct tm *t;
	struct timeval tv;

	im.ft = (struct fit_tag *) malloc(sizeof(struct fit_tag));
	im.ft->next = NULL;
	ct_read_header_file(&im, local_config_file);
	gettimeofday(&tv, NULL);
	t = gmtime( (time_t *) &tv.tv_sec);
	im.tjd = julian_date((short int)(t->tm_year + 1900), 
			     (short int)(t->tm_mon + 1), 
			     (short int)t->tm_mday, 
			     (double)t->tm_hour + 
			     (((double)t->tm_sec)/60.0 + 
			      ((double)t->tm_min))/60.0);
	ct_precess(im.equinox, im.tjd, 
		   strdeg2double(im.ra_cat),
		   strdeg2double(im.dec_cat), ra, dec);
	
	fits_tag_free(im.ft);

	return 0;
}


int ct_read_save_file(int cam, char *filename)
{
	int len ;
	int i;
	int max_count = 1;
	int *data;
	int status;
	char str_tmp[80];
	struct image_2d im;
	struct tm *t;
	double rar, decr;
	double ra1, dec1;

	im.ft = (struct fit_tag *) malloc(sizeof(struct fit_tag));
	im.ft->next = NULL;

	if ( cc_act_rcv_setup(cam, &im.s) < 0) {
		PVERB(PWARN, " Error reading image setup.\n");
		return CT_IM_SETUP;
	}
	len = im.s.naxis2 * im.s.naxis1;
	if ( im.s.naxis2 > CCD_Y_LEN || im.s.naxis2 <= 0 ||  /* 384 */
	     im.s.naxis1 > CCD_X_LEN || im.s.naxis1 <= 0 ) { /* 576 */
		PVERB(PWARN, " invalid dimension %d x %d data image\n",
		      im.s.naxis2, im.s.naxis1);
		return CT_IM_DIM;
	}
	data = (int *) malloc(len * sizeof(int));
	PVERB(PINFO, " geting %d x %d data image: ", im.s.naxis2, im.s.naxis1);
	if ( cc_act_rcv_data(cam, data, &len) < 0) {
		PVERB(PERR, " Error reading image.\n");
		free(data);
		return CT_IM_READ;
	}
	im.datamin = *(data+2);
	im.datamax = *data;
	for (i=1;i<len;i++) {
		/* Skip over scan region: */
		if( (i % im.s.naxis2) != 0 && ((i-1) % im.s.naxis2) != 0 ) {
			if( *(data+i) < im.datamin) im.datamin = *(data+i);
			else if(*(data+i) > im.datamax) {
				im.datamax = *(data+i);
				max_count = 1;
			}else if (*(data+i) == im.datamax) max_count++;
		}
	}
	/* FIXME: this printf ... */
	printf("DATAMIN: %d, DATAMAX: %d (in %d pixels)\n",
	       im.datamin, im.datamax, max_count);
	im.data = data;
	sprintf(str_tmp,
		"stari camera control v %s, (c) Federico A. Bareilles (License GPL)", VERSION);
	comment_push(im.ft, str_tmp);
	fits_tag_push(im.ft, "SOFT_VER", VERSION, "Software version number");
	im.ra_cat[0] = im.dec_cat[0] = 0;
	sprintf(im.filter, "-");
	ct_read_header_file(&im, GLOBAL_CONFIG_FILE);
	ct_read_header_file(&im, local_config_file);
	t = gmtime( (time_t *) &im.s.tv.tv_sec);
	sprintf(im.date_obs,"%04d-%02d-%02d", t->tm_year + 1900, 
		t->tm_mon + 1, t->tm_mday);
	sprintf(im.time_obs,"%02d:%02d:%02d", t->tm_hour, 
		t->tm_min, t->tm_sec);
	im.tjd = julian_date((short int)(t->tm_year + 1900), 
			     (short int)(t->tm_mon + 1), 
			     (short int)t->tm_mday, 
			     (double)t->tm_hour + 
			     (((double)t->tm_sec)/60.0 + 
			      ((double)t->tm_min))/60.0);
	sidereal_time(im.tjd, 0.0, 0.0, &im.gst);
	im.lst = mod24(im.gst + im.longitude / 15.0);
	ct_precess(im.equinox, im.tjd, 
		   strdeg2double(im.ra_cat), 
		   strdeg2double(im.dec_cat), &ra1, &dec1);
	double2gms_str(im.ra, ra1);
	double2gms_str(im.dec, dec1);
	im.ha = mod24(im.lst - ra1);	
	equ2hor (im.tjd, DELTA_T, 0.0, 0.0,
		 &geo_loc,
		 ra1, dec1, 0,
		 &im.z, &im.azimuth, &rar, &decr);
	im.azimuth += 180.0;
	while(im.azimuth > 360.0) im.azimuth -= 360.0;
	im.airmas = airmas(im.z * _DEG2RAD);

	status = image_2d_save(filename, &im);

	free(data);
	fits_tag_free(im.ft);
	if (status != 0) {
		PVERB(PWARN, " Fail.\n");
		fits_report_error(stdout, status);
		return CT_IM_FITS_ERR;
	}

	return CT_IM_OK;
}


int ct_read_header_file(struct image_2d *im, char *filename)
{
	FILE *fp;
	char line[LINE_LEN_CONF];
	char *sline, *tag, *data, *comment;

	fp = fopen(filename, "r");
	
	if( fp == NULL ){
                SYSLOG(LOG_ERR,
                       "ct_read_header_file: Error opening file: %s",
		       filename);
                return -1;
        }
	
	memset(line, 0, sizeof(line));
	while( fgets(line, sizeof(line), fp)){
		if(line[strlen(line)-1] == '\n')
			line[strlen(line)-1] = 0;
		sline = stripwhite(line);
		if (parse_line(sline, &tag, &data, &comment) == 0){
			if( memcmp(tag,"OBJECT", 6) == 0 ){
				sprintf(im->object,"%s", data);
			}else if( memcmp(tag,"OBSERVER", 8) == 0 ){
				sprintf(im->observer, "%s", data);
			}else if( memcmp(tag,"OBSERVAT", 8) == 0 ){
				sprintf(im->observat, "%s", data);
			}else if( memcmp(tag,"TELESCOP", 8) == 0 ){
				sprintf(im->telescope, "%s", data);
			}else if( memcmp(tag,"INSTRUME", 8) == 0 ){
				sprintf(im->instrument, "%s", data);

			}else if( memcmp(tag,"PIXSCALE", 8) == 0 ){
				im->pixscale = atof(data);
			}else if( memcmp(tag,"RA", 2) == 0 ){
				sprintf(im->ra_cat, "%s", data);
			}else if( memcmp(tag,"DEC", 3) == 0 ){
				sprintf(im->dec_cat, "%s", data);
			}else if( memcmp(tag,"EQUINOX", 7) == 0 ){
				im->equinox = atof(data);
			}else if( memcmp(tag,"FILTER", 6) == 0 ){
				sprintf(im->filter, "%s", data);
			}else if( memcmp(tag,"LATITUDE", 8) == 0 ){
				im->latitude = strdeg2double(data);
			}else if( memcmp(tag,"LONGITUD", 8) == 0 ){
				im->longitude = strdeg2double(data);
			}else if( memcmp(tag,"ALTITUDE", 8) == 0 ){
				im->altitude = strdeg2double(data);
			}else if( memcmp(tag,"COMMENT", 7) == 0 ){
				comment_push(im->ft, data);
			}else{
				if(comment != NULL)
					fits_tag_push(im->ft, tag, data, 
						      comment);
				else
					fits_tag_push(im->ft, tag, data, "");
			}
		}
		memset(line, 0, sizeof(line));
	}
	fclose(fp);

	return 0;
}


int ct_precess(double from_date, double to_date,
	    double ra0,  double dec0,
	    double *ra1, double *dec1 )
{
	cat_entry star0 = {"SRC", "start", 1, ra0, dec0, 0.0, 0.0, 0.0, 0.0};
	cat_entry star1;

	transform_cat (3, 
		       from_date, &star0,
		       to_date, "nc", &star1);
	*ra1 = star1.ra;
	*dec1 = star1.dec;

	return 0;
}

